<?php

namespace FiloBlu\Storelocator\Model;

use Magento\Framework\Exception\LocalizedException as CoreException;
use Magento\Framework\Registry;

class Stores extends \Magento\Framework\Model\AbstractModel {

    protected $storesCollectionFactory;

    /**
     * Store manager
     *
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    protected $_googleApi;
    protected $config = null;
    protected $allowedCountryModel;
    protected $request;

    public function __construct(
        \FiloBlu\Storelocator\Model\ResourceModel\Stores\CollectionFactory $storesCollectionFactory, \Magento\Store\Model\StoreManagerInterface $_storeManager, \FiloBlu\Storelocator\Model\Google\Api $googleApi, \Magento\Framework\App\Config\ScopeConfigInterface $config, \Magento\Framework\Model\Context $context, \Magento\Framework\Registry $registry,\Magento\Framework\App\Request\Http $request,\Magento\Directory\Model\AllowedCountries $allowedCountryModel, \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null, \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null, array $data = []
    ) {
        $this->storesCollectionFactory = $storesCollectionFactory;
        $this->_storeManager = $_storeManager;
        $this->_googleApi = $googleApi;
        $this->config = $config;
        $this->allowedCountryModel = $allowedCountryModel;
        $this->request = $request;

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct() {
        $this->_init('FiloBlu\Storelocator\Model\ResourceModel\Stores');
    }

    public function getAvailableStatuses() {
        return array(
            0 => "Disabled",
            1 => "Enabled"
        );
    }

    public function beforeSave() {
        if (is_array($this->getData('store_ids'))) {
            $this->setData('store_ids', implode(',', $this->getData('store_ids')));
        }
        return parent::beforeSave();
    }

    public function beforeDelete() {
        if ($this->getGmbLocationName() && $this->config->getValue('filoblu_storelocator_section/google_my_business/gmb_enabled')) {
            $this->_googleApi->deleteLocation($this->getGmbLocationName());
        }
        return parent::beforeDelete();
    }

    public function getAllStores() {
        $params = $this->request->getParams();

        $collection = $this->storesCollectionFactory->create();
        $collection->addFieldToFilter('enabled', ["eq" => 1]);
        $collection->addStoreFilter($this->_storeManager->getStore()->getId());

        if (isset($params["pickup_in_store"]) ? (bool)$params["pickup_in_store"] : false) {
            $collection->addFieldToFilter('country', ["in" => $this->allowedCountryModel->getAllowedCountries()]);
            $collection->addFieldToFilter('is_enabled_for_pickup_in_store', ["eq" => 1]);
        }

        $collection->setOrder('country', 'ASC');
        $collection->setOrder('city', 'ASC');

        return $collection;
    }

    public function getAllLatLngByCountry($country) {
        $collection = $this->getAllStores();
        $collection->addFieldToFilter('country', ["eq" => $country]);
        $collection->addFieldToSelect(array('latitude', 'longitude'));

        return $collection;
    }

    public function getAllLatLngByCity($city,$country = null) {
        $collection = $this->getAllStores();
        $collection->addFieldToFilter('city', ["eq" => $city]);
        if ($country) {
            $collection->addFieldToFilter('country', ["eq" => $country]);
        }
        $collection->addFieldToSelect(array('latitude', 'longitude'));

        return $collection;
    }

    public function getCitiesByCountry($country) {
        $collection = $this->getAllStores();
        $collection->addFieldToSelect('city');
        $collection->addFieldToFilter('country', ["eq" => $country]);

        $collection->getSelect()->group('city');

        return $collection;
    }

    public function getCitiesByCountryHTML($country) {
        $collection = $this->getCitiesByCountry($country);

        $citiesArray = [];
        $citiesHTML = '';

        foreach ($collection->getData() as $city) {
            if ($city["city"] != '') {
                array_push($citiesArray, ['label' => $city["city"], 'value' => $city["city"]]);
            }
        }

        // Obtain a list of columns
        foreach ($citiesArray as $key => $row) {
            $label[$key] = $row['label'];
        }

        // Sort the data with volume descending, edition ascending
        // Add $data as the last parameter, to sort by the common key
        array_multisort($label, SORT_ASC, $citiesArray);

        array_unshift($citiesArray, ['value' => '', 'label' => __('Select a city...')->__toString()]);

        foreach ($citiesArray as $key => $row) {
            $citiesHTML .= '<option value="' . $row['value'] . '">' . $row['label'] . '</option>';
        }

        return $citiesHTML;
    }

    public function getAllAvailableCountries() {
        $collection = $this->getAllStores();
        $collection->addFieldToSelect('country');

        $collection->getSelect()->group('country');

        return $collection;
    }

    public function getAllAvailableCountriesHTML() {
        $collection = $this->getAllAvailableCountries();
        $countryModel = \Magento\Framework\App\ObjectManager::getInstance()->get('\Magento\Directory\Model\Country');

        $countriesArray = [];
        $countriesHTML = '';

        foreach ($collection->getData() as $country) {
            if ($country["country"] != '') {
                $countryData = $countryModel->loadByCode($country["country"]);
                array_push($countriesArray, ['label' => $countryData->getName(), 'value' => $country["country"]]);
            }
        }

        // Obtain a list of columns
        foreach ($countriesArray as $key => $row) {
            $label[$key] = $row['label'];
        }

        // Sort the data with volume descending, edition ascending
        // Add $data as the last parameter, to sort by the common key
        if (count($countriesArray)) {
          array_multisort($label, SORT_ASC, $countriesArray);
        }

        array_unshift($countriesArray, ['value' => '', 'label' => __('Select a country...')->__toString()]);

        foreach ($countriesArray as $key => $row) {
            $countriesHTML .= '<option value="' . $row['value'] . '">' . $row['label'] . '</option>';
        }

        return $countriesHTML;
    }

    public function getStoresByCurrentZoom($lat1, $lng1, $lat2, $lng2) {
        $collection = $this->getAllStores();
        $collection->addFieldToFilter('latitude', ["from" => $lat1, "to" => $lat2]);
        $collection->addFieldToFilter('longitude', ["from" => $lng1, "to" => $lng2]);

        $modelWT = \Magento\Framework\App\ObjectManager::getInstance()->get('FiloBlu\Storelocator\Model\WorkingTimes');

        foreach ($collection as $store) {

            if ($store->getImage()) {
                $store->setImageUrl($this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . "filoblu/storelocator/stores/image" . $store->getImage());
            }

            //book an appointment
            if ($store->getUrlKey()) {
                $store->setShopViewUrl($this->_storeManager->getStore()->getBaseUrl() . $store->getUrlKey());
            }
            else {
                $store->setShopViewUrl($this->_storeManager->getStore()->getBaseUrl() . 'storelocator/index/shop/id/' . $store->getStoreId());
            }


            //tags
            $tags = $this->getResource()->getTagsIdByStore($store->getStoreId());
            $store->setTagsId($tags);

            $tagsName = $this->getResource()->getTagsNameByStore($store->getStoreId());
            $store->setTagsName($tagsName);

            // image gallery
            $images = $this->getResource()->getImagesStore($store->getStoreId());
            $editImages = [];

            foreach ($images as $key => $img) {
                if ($img["image"]) {
                    array_push($editImages, [
                        "title" => $img["title"],
                        "description" => $img["description"],
                        "imageUrl" => $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . "filoblu/storelocator/stores/image" . $img["image"],
                            ]
                    );
                }
            }

            $store->setImageGallery($editImages);

            // working times
            $objWT = $modelWT->loadByStoreId($store->getStoreId());

            if ($objWT->count() == 0) {
                $store->setHasWorkingTimes(false);
            } else {

                $editWorkingTimes = [];
                $checkNothingEdit = false;
                foreach ($objWT as $wt) {
                    $timeFrom = explode(":", explode(" ", $wt["from"])[1]);
                    $fromHH = $timeFrom[0];
                    $fromMM = $timeFrom[1];

                    $wt["from"] = $fromHH . ':' . $fromMM;

                    $timeTo = explode(":", explode(" ", $wt["to"])[1]);
                    $toHH = $timeTo[0];
                    $toMM = $timeTo[1];

                    $wt["to"] = $toHH . ':' . $toMM;

                    array_push($editWorkingTimes, [
                        "day" => $wt["day"],
                        "from" => $wt["from"],
                        "to" => $wt["to"],
                        "closed" => (bool) $wt["closed"],
                        "period" => $wt["period"]
                            ]
                    );

                    if (($fromHH . $fromMM . $toHH . $toMM != '00000000')) {
                        $checkNothingEdit = true;
                    }
                }

                $store->setHasWorkingTimes($checkNothingEdit);
                $store->setWorkingTimes($editWorkingTimes);
            }

        }

        return $collection;
    }

    /**
     * @return array|mixed
     */
    public function getOldTags() {
        if (!$this->getStoreId()) {
            return array();
        }
        $array = $this->getData('old_tags');


        if (!$array) {

            $array = $this->getResource()->getOldTags($this);
            $this->setData('old_tags', $array);
        }

        return $array;
    }

    /**
     * @return array|mixed
     */
//    public function getOldImages()
//    {
//        if (!$this->getStoreId()) {
//            return array();
//        }
//        $array = $this->getData('old_images');
//
//
//        if (!$array) {
//
//            $array = $this->getResource()->getOldImages($this);
//            $this->setData('old_images', $array);
//        }
//
//        return $array;
//    }

    /**
     * @return array|mixed
     */
    public function getImagesPosition() {
        if (!$this->getStoreId()) {
            return array();
        }
        $array = $this->getData('images_position');
        if (is_null($array)) {
            $array = $this->getResource()->getImagesPosition($this);
            $this->setData('images_position', $array);
        }

        return $array;
    }

}
